#![allow(dead_code, unused_macros)]
use std::{
    net::{Shutdown, TcpStream},
    process::Stdio,
};
type Disks = Vec<Vec<String>>;

macro_rules! count_tts {
    () => { 0 };
    ($odd:tt $($a:tt $b:tt)*) => { (count_tts!($($a)*) << 1) | 1 };
    ($($a:tt $even:tt)*) => { count_tts!($($a)*) << 1 };
}

macro_rules! read {
    ($type:ty, $($prompts:literal),+) => {{
        use std::io::{stdout, stdin, Write};
        let r#type = match stringify!($type) {
            "bool" => "Boolean",
            "usize" => "Integer",
            "String" => "String",
             _=> panic!("\nunkown type {}", stringify!($type))
        };
        loop {
            let mut result = String::new();
            $(println!("\n{}", $prompts);)*
            print!("> ");
            let _ = stdout().flush();
            stdin()
                .read_line(&mut result)
                .expect("Did not enter a correct string");
            if let Some('\n') = result.chars().next_back() {
                result.pop();
            }
            let results = result.parse::<$type>();
            match results {
                Ok(v) => break v,
                Err(_) => println!("\nInput does not match expected type: {}.", r#type),
            }
        }
    }};
}

/// Currently does not work for standalone long dash
macro_rules! cmd {
    (@command $token:tt) => {
        stringify!($token).replace(" - ", "-")
    };
    (@args $($token:tt)+) => {{
        let args = [$(stringify!($token)),+];

        let mut prv_val: &'static str = "";
        args.iter()
    .map(|token| {
            let mut results: Option<String> = None;
            if (prv_val != "-" || prv_val != "--") && *token != "-" {
                results = Some(format!("{}", token.replace('"', "")));
            }
            if prv_val == "-" && *token != "-" {
                results = Some(format!("-{}", *token));
            }
            if prv_val == "--" && *token != "-" {
                results = Some(format!("--{}", *token));
            }
            prv_val = if prv_val == "-" && *token == "-" {
                "--"
            } else {
                token
            };
            match results {
                Some(val) => val,
                None => String::from(""),
            }
        })
        .filter(|item| !item.is_empty())
        .collect::<Vec<String>>()
    }};
    ($command:expr) => {{
        let command = cmd!(@command $command);
        std::process::Command::new(command)

    }};
    ($command:expr => $($args:tt)+)  => {{
        let command = cmd!(@command $command);
        #[allow(unused_mut,)]
        let mut args = cmd!(@args $($args) *);
        std::process::Command::new(command).args(args)
    }};
}

fn is_online() -> bool {
    let archlinux_org = "95.217.163.246:80";
    let ping = TcpStream::connect(archlinux_org);
    match ping {
        Ok(val) => {
            let _ = val.shutdown(Shutdown::Both);
            true
        }
        Err(_) => panic!("Connect to a network then rerun script"),
    }
}

fn is_uefi() -> bool {
    let bit = std::fs::read_to_string("/sys/firmware/efi/fw_platform_size");
    match bit {
        Ok(_) => true,
        _ => false,
    }
}

fn get_disks() -> Disks {
    let process = std::process::Command::new("lsblk")
        .args([
            "--paths",
            "--output",
            "NAME,SIZE",
            "--noheadings",
            "--nodeps",
        ])
        .output();
    let output = String::from_utf8(process.unwrap().stdout)
        .unwrap()
        .trim()
        .to_owned();

    output
        .split("\n")
        .map(|disk| {
            disk.split_whitespace()
                .map(|value| value.to_owned())
                .collect()
        })
        .collect()
}

fn print_disks(disks: &Disks) {
    for (i, disk) in disks.iter().enumerate() {
        println!("{}\t{}\t{}", i, disk[0], disk[1]);
    }
}

#[derive(strum::AsRefStr)]
enum DiskLabel {
    GPT,
    MBR,
}

fn fdisk(dev: &String, label: DiskLabel, partitions: Vec<&'static str>) {
    let label = format!("label: {}", label.as_ref());

    let mut process = std::process::Command::new("sfdisk")
        // .arg("--quiet")
        .arg(dev)
        .stdin(Stdio::piped())
        .spawn()
        .expect("Command failed to spawn");

    std::process::Command::new("echo")
        .arg([label, partitions.join("\n")].join("\n"))
        .stdout(process.stdin.take().unwrap()) // Converted into a Stdio here
        .spawn()
        .expect("failed echo command");

    let _ = process.wait();
}

fn mkfs_fat(dev: &String, part_num: i32, size: i32, label: &'static str) {
    let _ = std::process::Command::new("mkfs.fat")
        .args(["-n", label])
        .args(["-F", &size.to_string()])
        .arg(format!("{}{}", dev, part_num))
        .spawn()
        .unwrap()
        .wait()
        .expect("Command failed to spawn");
}

fn mkfs_btrfs(dev: &String, part_num: i32, label: &'static str) {
    let _ = std::process::Command::new("mkfs.btrfs")
        .args(["--label", label])
        .arg("-f")
        .arg(format!("{}{}", dev, part_num))
        .spawn()
        .unwrap()
        .wait()
        .expect("Command failed to spawn");
}

fn mount(dev: &String, part_num: i32, path: &'static str, options: Option<Vec<&'static str>>) {
    let _ = std::process::Command::new("mount")
        .arg("--mkdir")
        .args(match options {
            Some(opts) => vec!["--options".to_string(), opts.join(",")],
            None => vec![],
        })
        .arg(format!("{dev}{part_num}"))
        .arg(path)
        .spawn()
        .unwrap()
        .wait()
        .expect("Command failed to spawn");
}

fn main() {
    if !nix::unistd::getuid().is_root() {
        panic!("Script must be ran as root")
    }
    is_online();
    let _is_uefi = is_uefi();
    let disks = get_disks();

    // Get info from user.
    print_disks(&disks);
    let root = read!(usize, "Which device do you want to be root drive?");

    let mut fmt_home = false;
    let home = if read!(
        bool,
        "Do you want to store home on another drive? (true|false)"
    ) {
        let home = read!(usize, "Which device do you want to be home drive");
        fmt_home = read!(bool, "Do you want to format home? (true|false)");
        home
    } else {
        root
    };
    let swap = read!(bool, "Do you want a swap space? (true|false)");
    let swap_size = if swap {
        read!(
            String,
            "Swap size. Default is 2g. (accepting k/m/g/e/p suffix)"
        )
    } else {
        String::from("2g")
    };

    // Act on user info.
    // -- format disks and mkfs
    {
        let boot = ",+1G,C12A7328-F81F-11D2-BA4B-00A0C93EC93B";
        let part_root = ",";

        fdisk(&disks[root][0], DiskLabel::GPT, vec![boot, part_root]);

        mkfs_fat(&disks[root][0], 1, 32, "boot");
        mkfs_btrfs(&disks[root][0], 2, "root");
    };

    if fmt_home {
        fdisk(&disks[home][0], DiskLabel::GPT, vec![","]);

        mkfs_btrfs(&disks[home][0], 1, "home");
    }

    // -- mount partitions
    mount(&disks[root][0], 2, "/mnt", None);
    mount(&disks[root][0], 1, "/mnt/boot", None);
    if swap {
        _ = cmd! {btrfs => subvolume create "/mnt/swap"}
            .spawn()
            .unwrap()
            .wait();
        _ = cmd! {btrfs => filesystem mkswapfile --uuid clear "/mnt/swap/file"}
            .args(["--size", &swap_size])
            .spawn()
            .unwrap()
            .wait();
        _ = cmd! {swapon => "/mnt/swap/file"}.spawn().unwrap().wait();
    }

    // Installation
    // -- Sort mirrors
    _ = cmd! {reflector => --save "/etc/pacman.d/mirrorlist" --sort rate --threads 24}
        .spawn()
        .unwrap()
        .wait();
    // -- Install essential packages
    _ = cmd! {pacstrap => -K "/mnt" base "linux-zen" "linux-firmware" micro}
        .spawn()
        .unwrap()
        .wait();

    // Configure the system
    // -- Fstab
    let fstab = cmd! {genfstab => -U "/mnt"}.output().unwrap();
    let _ = std::fs::write("/mnt/etc/fstab", fstab.stdout);
    // -- Chroot
    let chroot = "#!/bin/env bash\nln -sf /usr/share/zoneinfo/America/Los_Angeles /etc/localtime\n\nhwclock --systohc\n\nsed -i 's/#en_US\\./en_US./' /etc/locale.gen\n\nlocale-gen\n\necho \"LANG=en_US.UTF-8\" >>/etc/locale.conf\n\necho \"zesktop\" >>/etc/hostname\n\nsed -i 's/#COMPRESSION=\"zstd\"/COMPRESSION=\"zstd\"/' /etc/mkinitcpio.conf\n\nprintf \"[Match]\\n\" >>/etc/systemd/network/20-ethernet.network\nprintf \"Name=en*\\n\" >>/etc/systemd/network/20-ethernet.network\nprintf \"Name=eth*\\n\" >>/etc/systemd/network/20-ethernet.network\nprintf \"\\n\" >>/etc/systemd/network/20-ethernet.network\nprintf \"[Network]\\n\" >>/etc/systemd/network/20-ethernet.network\nprintf \"DHCP=yes\\n\" >>/etc/systemd/network/20-ethernet.network\n\necho \"password\npassword\" | passwd\n";
    _ = std::fs::write("/mnt/chroot.bash", chroot);
    let _ = cmd! {arch-chroot => "/mnt" bash -c "chmod +x /chroot.bash; /chroot.bash"}
        .spawn()
        .unwrap()
        .wait();
}
